@@ -33,7 +33,7 @@ def _more_methods_for_backend(tnbackend: Any) -> None:
 
     def copy(self: Any, a: Tensor) -> Tensor:
         """
-        Return the expm of ``a``, matrix exponential.
+        Return the copy of ``a``, matrix exponential.
 
         :param a: tensor in matrix form
         :type a: Tensor
@@ -46,7 +46,7 @@ def _more_methods_for_backend(tnbackend: Any) -> None:
 
     def expm(self: Any, a: Tensor) -> Tensor:
         """
-        Return the copy of tensor ''a''.
+        Return the expm of tensor ''a''.
 
         :param a: tensor in matrix form
         :type a: Tensor
@@ -188,6 +188,19 @@ def _more_methods_for_backend(tnbackend: Any) -> None:
             "Backend '{}' has not implemented `atan2`.".format(self.name)
         )
 
+    def atanh(self: Any, a: Tensor) -> Tensor:
+        """
+        Return the atanh of a tensor ``a``.
+
+        :param a: tensor in matrix form
+        :type a: Tensor
+        :return: atanh of ``a``
+        :rtype: Tensor
+        """
+        raise NotImplementedError(
+            "Backend '{}' has not implemented `atanh`.".format(self.name)
+        )
+
     def cosh(self: Any, a: Tensor) -> Tensor:
         """
         Return the cosh of a tensor ``a``.
@@ -255,8 +268,6 @@ def _more_methods_for_backend(tnbackend: Any) -> None:
             "Backend '{}' has not implemented `abs`.".format(self.name)
         )
 
-    # TODO(@refraction-ray): abs docstring doesn't get registered in the doc
-
     def kron(self: Any, a: Tensor, b: Tensor) -> Tensor:
         """
         Return the kronecker product of two matrices ``a`` and ``b``.
@@ -1161,6 +1172,60 @@ def _more_methods_for_backend(tnbackend: Any) -> None:
             "Backend '{}' has not implemented `is_sparse`.".format(self.name)
         )
 
+    def device(self: Any, a: Tensor) -> str:
+        """
+        get the universal device str for the tensor, in the format of tf
+
+        :param a: the tensor
+        :type a: Tensor
+        :return: device str where the tensor lives on
+        :rtype: str
+        """
+        raise NotImplementedError(
+            "Backend '{}' has not implemented `device`.".format(self.name)
+        )
+
+    def device_move(self: Any, a: Tensor, dev: Any) -> Tensor:
+        """
+        move tensor ``a`` to device ``dev``
+
+        :param a: the tensor
+        :type a: Tensor
+        :param dev: device str or device obj in corresponding backend
+        :type dev: Any
+        :return: the tensor on new device
+        :rtype: Tensor
+        """
+        raise NotImplementedError(
+            "Backend '{}' has not implemented `device_move`.".format(self.name)
+        )
+
+    def _dev2str(self: Any, dev: Any) -> str:
+        """
+        device object to universal dev str
+
+        :param dev: device object
+        :type dev: Any
+        :return: dev str
+        :rtype: str
+        """
+        raise NotImplementedError(
+            "Backend '{}' has not implemented `_dev2str`.".format(self.name)
+        )
+
+    def _str2dev(self: Any, str_: str) -> Any:
+        """
+        device object to universal dev str
+
+        :param str_: dev str
+        :type str_: str
+        :return: device object
+        :rtype: Any
+        """
+        raise NotImplementedError(
+            "Backend '{}' has not implemented `_str2dev`.".format(self.name)
+        )
+
     def cond(
         self: Any,
         pred: bool,
