@@ -170,6 +170,11 @@ class Gate(tn.Node):  # type: ignore
             f"{sp}edges: {edges_out})"
         )
 
+    def copy(self, conjugate: bool = False) -> "Gate":
+        result = super().copy(conjugate=conjugate)
+        result.__class__ = Gate
+        return result  # type: ignore
+
 
 def num_to_tensor(*num: Union[float, Tensor], dtype: Optional[str] = None) -> Any:
     r"""
@@ -679,6 +684,7 @@ def any_gate(unitary: Tensor, name: str = "any") -> Gate:
     if isinstance(unitary, Gate):
         unitary.tensor = backend.cast(unitary.tensor, dtypestr)
         return unitary
+    unitary = backend.cast(unitary, dtypestr)
     unitary = backend.reshape2(unitary)
     # nleg = int(np.log2(backend.sizen(unitary)))
     # unitary = backend.reshape(unitary, [2 for _ in range(nleg)])
@@ -751,6 +757,20 @@ ryy_gate = partial(exp1_gate, unitary=_yy_matrix)
 
 
 def multicontrol_gate(unitary: Tensor, ctrl: Union[int, Sequence[int]] = 1) -> Operator:
+    r"""
+    Multicontrol gate. If the control qubits equal to ``ctrl``, :math:`U` is applied to the target qubits.
+
+    E.g., ``multicontrol_gate(tc.gates._zz_matrix, [1, 0, 1])`` returns a gate of 5 qubits,
+        where the last 2 qubits are applied :math:`ZZ` gate,
+        if the first 3 qubits are :math:`\ket{101}`.
+
+    :param unitary: input unitary :math:`U`
+    :type unitary: Tensor
+    :param ctrl: control bit sequence
+    :type ctrl: Union[int, Sequence[int]]
+    :return: Multicontrol Gate
+    :rtype: Operator
+    """
     if isinstance(unitary, tn.Node):
         unitary = unitary.tensor
     unitary = backend.reshapem(unitary)
